package org.eclipse.corona.mylar;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.FileNotFoundException;

import org.eclipse.mylar.context.core.ContextCorePlugin;
import org.eclipse.mylar.context.core.IMylarContext;
import org.eclipse.mylar.internal.context.core.CompositeContext;
import org.eclipse.mylar.context.core.IMylarElement;
import org.eclipse.mylar.internal.context.core.MylarContextManager;
import org.eclipse.mylar.monitor.usage.MylarUsageMonitorPlugin;
import org.eclipse.mylar.context.core.InteractionEvent;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;

public class Mylar implements IMylar
{
	private BundleContext m_BundleContext = null;
	private ServiceRegistration m_SrvRegistration = null;
	
	/**
	 * Class constructor.  Performs basic initialization for the class instance.
	 * 
	 * @param context				Bundle context.
	 * @param bServerTopicManager	<code>true</code> indicates topic manager is running on a server.
	 * 								<code>false</code> indicate topic manager is running on
	 * 								a client.
	 */
	public Mylar(BundleContext context)
	{
		m_BundleContext = context;
		m_SrvRegistration = context.registerService(IMylar.class.getName(), this, null);
	}
	
	/**
	 * Close the CollaborationEventRouter.
	 * 
	 * The service that was registered will be unregistered.
	 */
	public void close()	
	{
		if (m_SrvRegistration != null)
		{
			m_SrvRegistration.unregister();
		}
		
		m_BundleContext = null;
	}
	
	/**
	 * 	Add a SELECTION event to the current Mylar context
	 * 
	 * @param	resourceName	Name of the resource being selected.
	 * @param	resourceKind	Kind of resource beinbg selected.
	 * @param	originId		Id of component selecting resource.
	 */
	public boolean addSelectionEvent(String resourceName, String resourceKind, String originId)
	{
		boolean bAdded = false;
		
		//
		//	Get contect manager
		//
		MylarContextManager contextManager = ContextCorePlugin.getContextManager();
		
		//
		//	Get current context
		//
		IMylarContext curContext = contextManager.getActiveContext();
		if (curContext != null)
		{
			//
			//	Set resource kind if not supplied
			//
			if ((resourceKind == null) || (resourceKind.length() == 0))
			{
				resourceKind = "*";
			}
		
			//
			//	Create interaction event
			//
			InteractionEvent interactionEvent = new InteractionEvent(InteractionEvent.Kind.SELECTION, resourceKind, resourceName, originId, "null");
			
			//
			//	Add event to context
			//
			contextManager.handleInteractionEvent(interactionEvent);
			bAdded = true;
		}

		return bAdded;
	}
	
	/**
	 * 	Add a MANIPULATION event to the current Mylar context
	 * 
	 * @param	resourceName	Name of the resource being manipluated.
	 * @param	resourceKind	Kind of resource being manipluated.
	 * @param	originId		Id of component manipluating resource.
	 */
	public boolean addManipluationEvent(String resourceName, String resourceKind, String originId)
	{
		boolean bAdded = false;
		
		//
		//	Get contect manager
		//
		MylarContextManager contextManager = ContextCorePlugin.getContextManager();
		
		//
		//	Get current context
		//
		IMylarContext curContext = contextManager.getActiveContext();
		if (curContext != null)
		{
			//
			//	Set resource kind if not supplied
			//
			if ((resourceKind == null) || (resourceKind.length() == 0))
			{
				resourceKind = "null";
			}
		
			//
			//	Create interaction event
			//
			InteractionEvent interactionEvent = new InteractionEvent(InteractionEvent.Kind.MANIPULATION, resourceKind, resourceName, originId, "null");
			
			//
			//	Add event to context
			//
			contextManager.handleInteractionEvent(interactionEvent);
			bAdded = true;
		}

		return bAdded;
	}
	
	/**
	 * 	Add a PROPAGATION event to the current Mylar context
	 * 
	 * @param	resourceName	Name of the resource being propgated.
	 * @param	resourceKind	Kind of resource being propagated.
	 * @param	originId		Id of component propgating the resource.
	 */
	public boolean addPropagationEvent(String resourceName, String resourceKind, String originId)
	{
		boolean bAdded = false;
		
		//
		//	Get contect manager
		//
		MylarContextManager contextManager = ContextCorePlugin.getContextManager();
		
		//
		//	Get current context
		//
		IMylarContext curContext = contextManager.getActiveContext();
		if (curContext != null)
		{
			//
			//	Set resource kind if not supplied
			//
			if ((resourceKind == null) || (resourceKind.length() == 0))
			{
				resourceKind = "null";
			}
			
			//
			//	Create interaction event
			//
			InteractionEvent interactionEvent = new InteractionEvent(InteractionEvent.Kind.PROPAGATION, resourceKind, resourceName, originId, "null");
			
			//
			//	Add event to context
			//
			contextManager.handleInteractionEvent(interactionEvent);
			bAdded = true;
		}

		return bAdded;
	}
	
	/**
	 * 	Add a COMMAND event to the current Mylar activity
	 * 
	 * @param	resourceName	Name of the resource being propgated.
	 * @param	resourceKind	Kind of resource being propagated.
	 * @param	originId		Id of component propgating the resource.
	 */
	public boolean addActivityCommmand(String resourceName, String resourceKind, String originId, String command)
	{
		boolean bAdded = false;
		
		//
		//	Get contect manager
		//
		MylarContextManager contextManager = ContextCorePlugin.getContextManager();
		
		//
		//	Get current context
		//
		IMylarContext curContext = contextManager.getActiveContext();
		if (curContext != null)
		{
			//
			//	Set resource kind if not supplied
			//
			if ((resourceKind == null) || (resourceKind.length() == 0))
			{
				resourceKind = "ContextContainer";
			}
			
			//
			//	Create interaction event
			//
			InteractionEvent interactionEvent = new InteractionEvent(InteractionEvent.Kind.COMMAND, resourceKind, resourceName, originId, "", command, 1f);
			
			//
			//	Add event to context
			//
			contextManager.handleActivityMetaContextEvent(interactionEvent);
			
			IMylarElement activeElement = contextManager.getActiveElement();
			bAdded = true;
		}

		return bAdded;
	}
	
	/**
	 * Retrieve the name of the currently active Mylar element.
	 * 
	 * @return	The name of the active Mylar element. <code>null</code>
	 * 			is return if no active Mylar element.
	 */
	public String getCurrentElement()
	{
		String curElementName = null;
		
		//
		//	Get contect manager
		//
		MylarContextManager contextManager = ContextCorePlugin.getContextManager();
		
		//
		//	Get current context
		//
		IMylarContext curContext = contextManager.getActiveContext();
		
		CompositeContext curCompositeContext = null;
		if (curContext instanceof CompositeContext)
		{
			curCompositeContext = (CompositeContext)curContext;
			IMylarElement curElement = curCompositeContext.getActiveNode();
			if (curElement != null)
			{
				curElementName = curElement.getHandleIdentifier();
			}
		}
		
		return curElementName;
	}
	
	public void UpLoadHistory()
	{
		MylarUsageMonitorPlugin mylarPlugin = MylarUsageMonitorPlugin.getDefault();
		File monitorFile = MylarUsageMonitorPlugin.getDefault().getMonitorLogFile();
		if (monitorFile != null)
		{
			//
			//	Stop monitoring
			//
			boolean bMonitorStopped = false;
			MylarUsageMonitorPlugin.getDefault().getInteractionLogger().stopMonitoring();
			bMonitorStopped = true;

			//
			//	Open the monitor file
			//
			try
			{
				InputStream reader = new FileInputStream(monitorFile);
			
				//
				//	read the monitor file
				//
				byte[] buffer = new byte[1000];
				int bytesRead = 0;
				StringBuffer monitorData = new StringBuffer();
				while ((bytesRead = reader.read(buffer)) != -1)
				{
					monitorData.append(new String(buffer, 0, bytesRead));
					buffer = new byte[1000];
				}
				
				reader.close();
				
				//
				//	Restart monitoring
				//
				MylarUsageMonitorPlugin.getDefault().getInteractionLogger().startMonitoring();
				bMonitorStopped = false;
				
				//
				//	If the monitor fiel contained any data, then send the file
				//	to the server
				//
				if ((monitorData != null) && (monitorData.length() != 0))
				{
					//
					//	TODO: Add code to send file
					//
					
				}
			}
			catch(Exception ex)
			{
				if (bMonitorStopped == true)
				{
					MylarUsageMonitorPlugin.getDefault().getInteractionLogger().startMonitoring();
					bMonitorStopped = false;
				}
			}
		}
	}
}
